@@ -31,6 +31,6 @@ struct aclk_query_threads {
 void aclk_query_threads_start(struct aclk_query_threads *query_threads, mqtt_wss_client client);
 void aclk_query_threads_cleanup(struct aclk_query_threads *query_threads);
 
-const char *aclk_query_get_name(aclk_query_type_t qt);
+const char *aclk_query_get_name(aclk_query_type_t qt, int unknown_ok);
 
 #endif //NETDATA_AGENT_CLOUD_LINK_H
