@@ -8,8 +8,6 @@
 #include "connection.h"
 #include "node_connection.h"
 #include "node_creation.h"
-#include "chart_config.h"
-#include "chart_stream.h"
 #include "alarm_config.h"
 #include "alarm_stream.h"
 #include "node_info.h"
