@@ -9,7 +9,7 @@ extern char *claiming_pending_arguments;
 extern struct config cloud_config;
 
 void claim_agent(char *claiming_arguments);
-char *is_agent_claimed(void);
+char *get_agent_claimid(void);
 void load_claiming_state(void);
 void load_cloud_conf(int silent);
 
