@@ -30,6 +30,7 @@
 #define NETDATA_CHART_PRIO_SYSTEM_IP                    501
 #define NETDATA_CHART_PRIO_SYSTEM_IPV6                  502
 #define NETDATA_CHART_PRIO_SYSTEM_PROCESSES             600
+#define NETDATA_CHART_PRIO_SYSTEM_PROCESS_STATES        601
 #define NETDATA_CHART_PRIO_SYSTEM_FORKS                 700
 #define NETDATA_CHART_PRIO_SYSTEM_ACTIVE_PROCESSES      750
 #define NETDATA_CHART_PRIO_SYSTEM_CTXT                  800
@@ -43,6 +44,7 @@
 #define NETDATA_CHART_PRIO_SYSTEM_ENTROPY              1000
 #define NETDATA_CHART_PRIO_SYSTEM_UPTIME               1000
 #define NETDATA_CHART_PRIO_CLOCK_SYNC_STATE            1100
+#define NETDATA_CHART_PRIO_CLOCK_STATUS                1105
 #define NETDATA_CHART_PRIO_CLOCK_SYNC_OFFSET           1110
 #define NETDATA_CHART_PRIO_SYSTEM_IPC_MSQ_QUEUES       1200 // freebsd only
 #define NETDATA_CHART_PRIO_SYSTEM_IPC_MSQ_MESSAGES     1201
@@ -193,11 +195,11 @@
 #define NETDATA_CHART_PRIO_ZFS_IO                     2700
 #define NETDATA_CHART_PRIO_ZFS_HITS                   2520
 #define NETDATA_CHART_PRIO_ZFS_DHITS                  2530
-#define NETDATA_CHART_PRIO_ZFS_DEMAND_DATA_HITS       2531
-#define NETDATA_CHART_PRIO_ZFS_PREFETCH_DATA_HITS     2532
-#define NETDATA_CHART_PRIO_ZFS_PHITS                  2540
-#define NETDATA_CHART_PRIO_ZFS_MHITS                  2550
-#define NETDATA_CHART_PRIO_ZFS_L2HITS                 2560
+#define NETDATA_CHART_PRIO_ZFS_DEMAND_DATA_HITS       2540
+#define NETDATA_CHART_PRIO_ZFS_PREFETCH_DATA_HITS     2550
+#define NETDATA_CHART_PRIO_ZFS_PHITS                  2560
+#define NETDATA_CHART_PRIO_ZFS_MHITS                  2570
+#define NETDATA_CHART_PRIO_ZFS_L2HITS                 2580
 #define NETDATA_CHART_PRIO_ZFS_LIST_HITS              2600
 #define NETDATA_CHART_PRIO_ZFS_HASH_ELEMENTS          2800
 #define NETDATA_CHART_PRIO_ZFS_HASH_CHAINS            2810
@@ -358,10 +360,8 @@
 
 #define NETDATA_CHART_PRIO_CHECKS                    99999
 
-#define NETDATA_CHART_PRIO_NETDATA_DISKSPACE        132020
 #define NETDATA_CHART_PRIO_NETDATA_TIMEX            132030
-#define NETDATA_CHART_PRIO_NETDATA_TC_CPU           135000
-#define NETDATA_CHART_PRIO_NETDATA_TC_TIME          135001
+#define NETDATA_CHART_PRIO_NETDATA_TC_TIME          1000100
 
 
 #endif //NETDATA_ALL_H
