@@ -27,6 +27,14 @@ struct iface {
     struct iface *next;
 };
 
+unsigned int calc_num_ifaces(struct iface *root) {
+    unsigned int num = 0;
+    for (struct iface *h = root; h; h = h->next) {
+        num++;
+    }
+    return num;
+}
+
 unsigned int read_iface_iflink(const char *prefix, const char *iface) {
     if(!prefix) prefix = "";
 
@@ -447,6 +455,25 @@ void detect_veth_interfaces(pid_t pid) {
         goto cleanup;
     }
 
+     unsigned int host_dev_num = calc_num_ifaces(host);
+     unsigned int cgroup_dev_num = calc_num_ifaces(cgroup);
+    // host ifaces == guest ifaces => we are still in the host namespace
+    // and we can't really identify which ifaces belong to the cgroup (e.g. Proxmox VM).
+    if (host_dev_num == cgroup_dev_num) {
+        unsigned int m = 0;
+        for (h = host; h; h = h->next) {
+            for (c = cgroup; c; c = c->next) {
+                if (h->ifindex == c->ifindex && h->iflink == c->iflink) {
+                    m++;
+                    break;
+                }
+            }
+        }
+        if (host_dev_num == m) {
+            goto cleanup;
+        }
+    }
+
     for(h = host; h ; h = h->next) {
         if(iface_is_eligible(h)) {
             for (c = cgroup; c; c = c->next) {
@@ -479,11 +506,21 @@ void call_the_helper(pid_t pid, const char *cgroup) {
     info("running: %s", command);
 
     pid_t cgroup_pid;
-    FILE *fp = mypopene(command, &cgroup_pid, environment);
-    if(fp) {
+    FILE *fp_child_input, *fp_child_output;
+
+    if(cgroup) {
+        (void)netdata_popen_raw_default_flags(&cgroup_pid, environment, &fp_child_input, &fp_child_output, PLUGINS_DIR "/cgroup-network-helper.sh", "--cgroup", cgroup);
+    }
+    else {
+        char buffer[100];
+        snprintfz(buffer, sizeof(buffer) - 1, "%d", pid);
+        (void)netdata_popen_raw_default_flags(&cgroup_pid, environment, &fp_child_input, &fp_child_output, PLUGINS_DIR "/cgroup-network-helper.sh", "--pid", buffer);
+    }
+
+    if(fp_child_output) {
         char buffer[CGROUP_NETWORK_INTERFACE_MAX_LINE + 1];
         char *s;
-        while((s = fgets(buffer, CGROUP_NETWORK_INTERFACE_MAX_LINE, fp))) {
+        while((s = fgets(buffer, CGROUP_NETWORK_INTERFACE_MAX_LINE, fp_child_output))) {
             trim(s);
 
             if(*s && *s != '\n') {
@@ -499,7 +536,7 @@ void call_the_helper(pid_t pid, const char *cgroup) {
             }
         }
 
-        mypclose(fp, cgroup_pid);
+        netdata_pclose(fp_child_input, fp_child_output, cgroup_pid);
     }
     else
         error("cannot execute cgroup-network helper script: %s", command);
@@ -643,8 +680,13 @@ int main(int argc, char **argv) {
     if(argc != 3)
         usage();
 
-    if(!strcmp(argv[1], "-p") || !strcmp(argv[1], "--pid")) {
-        pid = atoi(argv[2]);
+    int arg = 1;
+    int helper = 1;
+    if (getenv("KUBERNETES_SERVICE_HOST") != NULL && getenv("KUBERNETES_SERVICE_PORT") != NULL)
+        helper = 0;
+
+    if(!strcmp(argv[arg], "-p") || !strcmp(argv[arg], "--pid")) {
+        pid = atoi(argv[arg+1]);
 
         if(pid <= 0) {
             errno = 0;
@@ -652,17 +694,17 @@ int main(int argc, char **argv) {
             return 2;
         }
 
-        call_the_helper(pid, NULL);
+        if(helper) call_the_helper(pid, NULL);
     }
-    else if(!strcmp(argv[1], "--cgroup")) {
-        char *cgroup = argv[2];
+    else if(!strcmp(argv[arg], "--cgroup")) {
+        char *cgroup = argv[arg+1];
         if(verify_path(cgroup) == -1) {
             error("cgroup '%s' does not exist or is not valid.", cgroup);
             return 1;
         }
 
         pid = read_pid_from_cgroup(cgroup);
-        call_the_helper(pid, cgroup);
+        if(helper) call_the_helper(pid, cgroup);
 
         if(pid <= 0 && !detected_devices) {
             errno = 0;
