@@ -137,10 +137,7 @@ getIntegerOption(
   return ((int)intvalue);
 }
 
-static int reset_job_metrics(const char *name, void *entry, void *data) {
-    (void)name;
-    (void)data;
-
+static int reset_job_metrics(const DICTIONARY_ITEM *item __maybe_unused, void *entry, void *data __maybe_unused) {
     struct job_metrics *jm = (struct job_metrics *)entry;
 
     jm->is_collected = 0;
@@ -175,8 +172,8 @@ struct job_metrics *get_job_metrics(char *dest) {
     return jm;
 }
 
-int collect_job_metrics(const char *name, void *entry, void *data) {
-    (void)data;
+int collect_job_metrics(const DICTIONARY_ITEM *item, void *entry, void *data __maybe_unused) {
+    const char *name = dictionary_acquired_item_name(item);
 
     struct job_metrics *jm = (struct job_metrics *)entry;
 
@@ -205,7 +202,7 @@ int collect_job_metrics(const char *name, void *entry, void *data) {
         printf("DIMENSION pending '' absolute 1 1\n");
         printf("DIMENSION held '' absolute 1 1\n");
         printf("DIMENSION processing '' absolute 1 1\n");
-        dictionary_del_having_write_lock(dict_dest_job_metrics, name);
+        dictionary_del(dict_dest_job_metrics, name);
     }
 
     return 0;
@@ -243,7 +240,7 @@ int main(int argc, char **argv) {
 
     errno = 0;
 
-    dict_dest_job_metrics = dictionary_create(DICTIONARY_FLAG_SINGLE_THREADED);
+    dict_dest_job_metrics = dictionary_create(DICT_OPTION_SINGLE_THREADED);
 
     // ------------------------------------------------------------------------
     // the main loop
