@@ -22,8 +22,11 @@ typedef uint8_t oomkill_ebpf_val_t;
 
 #define NETDATA_OOMKILL_CHART "oomkills"
 
+// Contexts
+#define NETDATA_CGROUP_OOMKILLS_CONTEXT "cgroup.oomkills"
+
 extern struct config oomkill_config;
-extern void *ebpf_oomkill_thread(void *ptr);
-extern void ebpf_oomkill_create_apps_charts(struct ebpf_module *em, void *ptr);
+void *ebpf_oomkill_thread(void *ptr);
+void ebpf_oomkill_create_apps_charts(struct ebpf_module *em, void *ptr);
 
 #endif /* NETDATA_EBPF_OOMKILL_H */
