@@ -5,7 +5,8 @@
 
 extern struct arcstats arcstats;
 
-// --------------------------------------------------------------------------------------------------------------------
+unsigned long long zfs_arcstats_shrinkable_cache_size_bytes = 0;
+
 // kstat.zfs.misc.arcstats
 
 int do_kstat_zfs_misc_arcstats(int update_every, usec_t dt) {
@@ -213,13 +214,18 @@ int do_kstat_zfs_misc_arcstats(int update_every, usec_t dt) {
     // missing mib: GETSYSCTL_SIMPLE("kstat.zfs.misc.arcstats.arc_need_free", mibs.arc_need_free, arcstats.arc_need_free);
     // missing mib: GETSYSCTL_SIMPLE("kstat.zfs.misc.arcstats.arc_sys_free", mibs.arc_sys_free, arcstats.arc_sys_free);
 
+    if (arcstats.size > arcstats.c_min) {
+        zfs_arcstats_shrinkable_cache_size_bytes = arcstats.size - arcstats.c_min;
+    } else {
+        zfs_arcstats_shrinkable_cache_size_bytes = 0;
+    }
+
     generate_charts_arcstats("freebsd.plugin", "zfs", show_zero_charts, update_every);
     generate_charts_arc_summary("freebsd.plugin", "zfs", show_zero_charts, update_every);
 
     return 0;
 }
 
-// --------------------------------------------------------------------------------------------------------------------
 // kstat.zfs.misc.zio_trim
 
 int do_kstat_zfs_misc_zio_trim(int update_every, usec_t dt) {
@@ -238,8 +244,6 @@ int do_kstat_zfs_misc_zio_trim(int update_every, usec_t dt) {
         return 1;
      } else {
 
-     // --------------------------------------------------------------------
-
         static RRDSET *st_bytes = NULL;
         static RRDDIM *rd_bytes = NULL;
 
@@ -261,13 +265,10 @@ int do_kstat_zfs_misc_zio_trim(int update_every, usec_t dt) {
 
             rd_bytes = rrddim_add(st_bytes, "TRIMmed", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
         }
-        else rrdset_next(st_bytes);
 
         rrddim_set_by_pointer(st_bytes, rd_bytes, bytes);
         rrdset_done(st_bytes);
 
-        // --------------------------------------------------------------------
-
         static RRDSET *st_requests = NULL;
         static RRDDIM *rd_successful = NULL, *rd_failed = NULL, *rd_unsupported = NULL;
 
@@ -291,7 +292,6 @@ int do_kstat_zfs_misc_zio_trim(int update_every, usec_t dt) {
             rd_failed      = rrddim_add(st_requests, "failed",      NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
             rd_unsupported = rrddim_add(st_requests, "unsupported", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
         }
-        else rrdset_next(st_requests);
 
         rrddim_set_by_pointer(st_requests, rd_successful,  success);
         rrddim_set_by_pointer(st_requests, rd_failed,      failed);
