@@ -70,11 +70,9 @@ int do_macos_sysctl(int update_every, usec_t dt) {
         do_uptime               = config_get_boolean("plugin:macos:sysctl", "system uptime", 1);
     }
 
-    RRDSET *st;
+    RRDSET *st = NULL;
 
-    int system_pagesize = getpagesize(); // wouldn't it be better to get value directly from hw.pagesize?
-    int i, n;
-    int common_error = 0;
+    int i;
     size_t size;
 
     // NEEDED BY: do_loadavg
@@ -96,7 +94,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
 
     // NEEDED BY: do_tcp...
     struct tcpstat tcpstat;
-    uint64_t tcps_states[TCP_NSTATES];
 
     // NEEDED BY: do_udp...
     struct udpstat udpstat;
@@ -221,8 +218,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
     // NEEDED BY: do_uptime
     struct timespec boot_time, cur_time;
 
-    // --------------------------------------------------------------------
-
     if (next_loadavg_dt <= dt) {
         if (likely(do_loadavg)) {
             if (unlikely(GETSYSCTL_BY_NAME("vm.loadavg", sysload))) {
@@ -250,7 +245,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                     rrddim_add(st, "load5", NULL, 1, 1000, RRD_ALGORITHM_ABSOLUTE);
                     rrddim_add(st, "load15", NULL, 1, 1000, RRD_ALGORITHM_ABSOLUTE);
                 }
-                else rrdset_next(st);
 
                 rrddim_set(st, "load1", (collected_number) ((double)sysload.ldavg[0] / sysload.fscale * 1000));
                 rrddim_set(st, "load5", (collected_number) ((double)sysload.ldavg[1] / sysload.fscale * 1000));
@@ -263,8 +257,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
     }
     else next_loadavg_dt -= dt;
 
-    // --------------------------------------------------------------------
-
     if (likely(do_swap)) {
         if (unlikely(GETSYSCTL_BY_NAME("vm.swapusage", swap_usage))) {
             do_swap = 0;
@@ -291,7 +283,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 rrddim_add(st, "free",    NULL, 1, 1048576, RRD_ALGORITHM_ABSOLUTE);
                 rrddim_add(st, "used",    NULL, 1, 1048576, RRD_ALGORITHM_ABSOLUTE);
             }
-            else rrdset_next(st);
 
             rrddim_set(st, "free", swap_usage.xsu_avail);
             rrddim_set(st, "used", swap_usage.xsu_used);
@@ -299,8 +290,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
         }
     }
 
-    // --------------------------------------------------------------------
-
     if (likely(do_bandwidth)) {
         mib[0] = CTL_NET;
         mib[1] = PF_ROUTE;
@@ -352,7 +341,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                     rrddim_add(st, "InOctets",  "received", 8, BITS_IN_A_KILOBIT, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutOctets", "sent",    -8, BITS_IN_A_KILOBIT, RRD_ALGORITHM_INCREMENTAL);
                 }
-                else rrdset_next(st);
 
                 rrddim_set(st, "InOctets", iftot.ift_ibytes);
                 rrddim_set(st, "OutOctets", iftot.ift_obytes);
@@ -361,8 +349,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
         }
     }
 
-    // --------------------------------------------------------------------
-
     // see http://net-snmp.sourceforge.net/docs/mibs/tcp.html
     if (likely(do_tcp_packets || do_tcp_errors || do_tcp_handshake || do_tcpext_connaborts || do_tcpext_ofo || do_tcpext_syscookies || do_ecn)) {
         if (unlikely(GETSYSCTL_BY_NAME("net.inet.tcp.stats", tcpstat))){
@@ -401,16 +387,13 @@ int do_macos_sysctl(int update_every, usec_t dt) {
 
                     rrddim_add(st, "InSegs", "received", 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutSegs", "sent", -1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "InSegs", tcpstat.tcps_rcvtotal);
                 rrddim_set(st, "OutSegs", tcpstat.tcps_sndtotal);
                 rrdset_done(st);
             }
 
-            // --------------------------------------------------------------------
-
             if (likely(do_tcp_errors)) {
                 st = rrdset_find_active_localhost("ipv4.tcperrors");
                 if (unlikely(!st)) {
@@ -433,8 +416,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                     rrddim_add(st, "InErrs", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "InCsumErrors", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "RetransSegs", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "InErrs", tcpstat.tcps_rcvbadoff + tcpstat.tcps_rcvshort);
                 rrddim_set(st, "InCsumErrors", tcpstat.tcps_rcvbadsum);
@@ -442,8 +424,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 rrdset_done(st);
             }
 
-            // --------------------------------------------------------------------
-
             if (likely(do_tcp_handshake)) {
                 st = rrdset_find_active_localhost("ipv4.tcphandshake");
                 if (unlikely(!st)) {
@@ -467,8 +447,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                     rrddim_add(st, "ActiveOpens", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "PassiveOpens", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "AttemptFails", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "EstabResets", tcpstat.tcps_drops);
                 rrddim_set(st, "ActiveOpens", tcpstat.tcps_connattempt);
@@ -477,8 +456,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 rrdset_done(st);
             }
 
-            // --------------------------------------------------------------------
-
             if (do_tcpext_connaborts == CONFIG_BOOLEAN_YES || (do_tcpext_connaborts == CONFIG_BOOLEAN_AUTO &&
                                                                (tcpstat.tcps_rcvpackafterwin ||
                                                                 tcpstat.tcps_rcvafterclose ||
@@ -508,7 +485,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                     rrddim_add(st, "TCPAbortOnMemory",  "nomemory",    1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "TCPAbortOnTimeout", "timeout",     1, 1, RRD_ALGORITHM_INCREMENTAL);
                 }
-                else rrdset_next(st);
 
                 rrddim_set(st, "TCPAbortOnData",    tcpstat.tcps_rcvpackafterwin);
                 rrddim_set(st, "TCPAbortOnClose",   tcpstat.tcps_rcvafterclose);
@@ -517,8 +493,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 rrdset_done(st);
             }
 
-            // --------------------------------------------------------------------
-
             if (do_tcpext_ofo == CONFIG_BOOLEAN_YES || (do_tcpext_ofo == CONFIG_BOOLEAN_AUTO &&
                                                         (tcpstat.tcps_rcvoopack ||
                                                          netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
@@ -542,14 +516,11 @@ int do_macos_sysctl(int update_every, usec_t dt) {
 
                     rrddim_add(st, "TCPOFOQueue", "inqueue",  1, 1, RRD_ALGORITHM_INCREMENTAL);
                 }
-                else rrdset_next(st);
 
                 rrddim_set(st, "TCPOFOQueue",   tcpstat.tcps_rcvoopack);
                 rrdset_done(st);
             }
 
-            // --------------------------------------------------------------------
-
             if (do_tcpext_syscookies == CONFIG_BOOLEAN_YES || (do_tcpext_syscookies == CONFIG_BOOLEAN_AUTO &&
                                                                (tcpstat.tcps_sc_sendcookie ||
                                                                 tcpstat.tcps_sc_recvcookie ||
@@ -578,7 +549,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                     rrddim_add(st, "SyncookiesSent",   "sent",     -1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "SyncookiesFailed", "failed",   -1, 1, RRD_ALGORITHM_INCREMENTAL);
                 }
-                else rrdset_next(st);
 
                 rrddim_set(st, "SyncookiesRecv",   tcpstat.tcps_sc_recvcookie);
                 rrddim_set(st, "SyncookiesSent",   tcpstat.tcps_sc_sendcookie);
@@ -586,9 +556,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 rrdset_done(st);
             }
 
-            // --------------------------------------------------------------------
-
-
 #if (defined __MAC_OS_X_VERSION_MIN_REQUIRED && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101100)
             if (do_ecn == CONFIG_BOOLEAN_YES || (do_ecn == CONFIG_BOOLEAN_AUTO &&
                                                  (tcpstat.tcps_ecn_recv_ce ||
@@ -616,7 +583,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                     rrddim_add(st, "InCEPkts", "CEP", 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "InNoECTPkts", "NoECTP", -1, 1, RRD_ALGORITHM_INCREMENTAL);
                 }
-                else rrdset_next(st);
 
                 rrddim_set(st, "InCEPkts", tcpstat.tcps_ecn_recv_ce);
                 rrddim_set(st, "InNoECTPkts", tcpstat.tcps_ecn_not_supported);
@@ -627,8 +593,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
         }
     }
 
-    // --------------------------------------------------------------------
-
     // see http://net-snmp.sourceforge.net/docs/mibs/udp.html
     if (likely(do_udp_packets || do_udp_errors)) {
         if (unlikely(GETSYSCTL_BY_NAME("net.inet.udp.stats", udpstat))) {
@@ -657,16 +621,13 @@ int do_macos_sysctl(int update_every, usec_t dt) {
 
                     rrddim_add(st, "InDatagrams", "received", 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutDatagrams", "sent", -1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "InDatagrams", udpstat.udps_ipackets);
                 rrddim_set(st, "OutDatagrams", udpstat.udps_opackets);
                 rrdset_done(st);
             }
 
-            // --------------------------------------------------------------------
-
             if (likely(do_udp_errors)) {
                 st = rrdset_find_active_localhost("ipv4.udperrors");
                 if (unlikely(!st)) {
@@ -693,8 +654,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
 #if (defined __MAC_OS_X_VERSION_MIN_REQUIRED && __MAC_OS_X_VERSION_MIN_REQUIRED >= 1090)
                     rrddim_add(st, "IgnoredMulti", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
 #endif
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "InErrors", udpstat.udps_hdrops + udpstat.udps_badlen);
                 rrddim_set(st, "NoPorts", udpstat.udps_noport);
@@ -710,8 +670,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
         }
     }
 
-    // --------------------------------------------------------------------
-
     if (likely(do_icmp_packets || do_icmpmsg)) {
         if (unlikely(GETSYSCTL_BY_NAME("net.inet.icmp.stats", icmpstat))) {
             do_icmp_packets = 0;
@@ -748,16 +706,12 @@ int do_macos_sysctl(int update_every, usec_t dt) {
 
                     rrddim_add(st, "InMsgs", "received", 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutMsgs", "sent", -1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "InMsgs", icmp_total.msgs_in);
                 rrddim_set(st, "OutMsgs", icmp_total.msgs_out);
-
                 rrdset_done(st);
 
-                // --------------------------------------------------------------------
-
                 st = rrdset_find_active_localhost("ipv4.icmp_errors");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
@@ -778,18 +732,14 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                     rrddim_add(st, "InErrors", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutErrors", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "InCsumErrors", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "InErrors", icmpstat.icps_badcode + icmpstat.icps_badlen + icmpstat.icps_checksum + icmpstat.icps_tooshort);
                 rrddim_set(st, "OutErrors", icmpstat.icps_error);
                 rrddim_set(st, "InCsumErrors", icmpstat.icps_checksum);
-
                 rrdset_done(st);
             }
 
-            // --------------------------------------------------------------------
-
             if (likely(do_icmpmsg)) {
                 st = rrdset_find_active_localhost("ipv4.icmpmsg");
                 if (unlikely(!st)) {
@@ -812,21 +762,17 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                     rrddim_add(st, "OutEchoReps", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "InEchos", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutEchos", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
-
-                    rrddim_set(st, "InEchoReps", icmpstat.icps_inhist[ICMP_ECHOREPLY]);
-                    rrddim_set(st, "OutEchoReps", icmpstat.icps_outhist[ICMP_ECHOREPLY]);
-                    rrddim_set(st, "InEchos", icmpstat.icps_inhist[ICMP_ECHO]);
-                    rrddim_set(st, "OutEchos", icmpstat.icps_outhist[ICMP_ECHO]);
+                }
 
+                rrddim_set(st, "InEchoReps", icmpstat.icps_inhist[ICMP_ECHOREPLY]);
+                rrddim_set(st, "OutEchoReps", icmpstat.icps_outhist[ICMP_ECHOREPLY]);
+                rrddim_set(st, "InEchos", icmpstat.icps_inhist[ICMP_ECHO]);
+                rrddim_set(st, "OutEchos", icmpstat.icps_outhist[ICMP_ECHO]);
                 rrdset_done(st);
             }
         }
     }
 
-    // --------------------------------------------------------------------
-
     // see also http://net-snmp.sourceforge.net/docs/mibs/ip.html
     if (likely(do_ip_packets || do_ip_fragsout || do_ip_fragsin || do_ip_errors)) {
         if (unlikely(GETSYSCTL_BY_NAME("net.inet.ip.stats", ipstat))) {
@@ -861,8 +807,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                     rrddim_add(st, "OutRequests", "sent", -1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "ForwDatagrams", "forwarded", 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "InDelivers", "delivered", 1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "OutRequests", ipstat.ips_localout);
                 rrddim_set(st, "InReceives", ipstat.ips_total);
@@ -871,8 +816,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 rrdset_done(st);
             }
 
-            // --------------------------------------------------------------------
-
             if (likely(do_ip_fragsout)) {
                 st = rrdset_find_active_localhost("ipv4.fragsout");
                 if (unlikely(!st)) {
@@ -895,8 +838,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                     rrddim_add(st, "FragOKs", "ok", 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "FragFails", "failed", -1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "FragCreates", "created", 1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "FragOKs", ipstat.ips_fragmented);
                 rrddim_set(st, "FragFails", ipstat.ips_cantfrag);
@@ -904,8 +846,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 rrdset_done(st);
             }
 
-            // --------------------------------------------------------------------
-
             if (likely(do_ip_fragsin)) {
                 st = rrdset_find_active_localhost("ipv4.fragsin");
                 if (unlikely(!st)) {
@@ -928,8 +868,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                     rrddim_add(st, "ReasmOKs", "ok", 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "ReasmFails", "failed", -1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "ReasmReqds", "all", 1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "ReasmOKs", ipstat.ips_fragments);
                 rrddim_set(st, "ReasmFails", ipstat.ips_fragdropped);
@@ -937,8 +876,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 rrdset_done(st);
             }
 
-            // --------------------------------------------------------------------
-
             if (likely(do_ip_errors)) {
                 st = rrdset_find_active_localhost("ipv4.errors");
                 if (unlikely(!st)) {
@@ -966,8 +903,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
 
                     rrddim_add(st, "InAddrErrors", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "InUnknownProtos", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "InDiscards", ipstat.ips_badsum + ipstat.ips_tooshort + ipstat.ips_toosmall + ipstat.ips_toolong);
                 rrddim_set(st, "OutDiscards", ipstat.ips_odropped);
@@ -980,8 +916,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
         }
     }
 
-    // --------------------------------------------------------------------
-
     if (likely(do_ip6_packets || do_ip6_fragsout || do_ip6_fragsin || do_ip6_errors)) {
         if (unlikely(GETSYSCTL_BY_NAME("net.inet6.ip6.stats", ip6stat))) {
             do_ip6_packets = 0;
@@ -1021,8 +955,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                     rrddim_add(st, "sent", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "forwarded", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "delivers", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "sent", ip6stat.ip6s_localout);
                 rrddim_set(st, "received", ip6stat.ip6s_total);
@@ -1031,8 +964,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 rrdset_done(st);
             }
 
-            // --------------------------------------------------------------------
-
             if (do_ip6_fragsout == CONFIG_BOOLEAN_YES || (do_ip6_fragsout == CONFIG_BOOLEAN_AUTO &&
                                                           (ip6stat.ip6s_fragmented ||
                                                            ip6stat.ip6s_cantfrag ||
@@ -1060,8 +991,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                     rrddim_add(st, "ok", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "failed", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "all", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "ok", ip6stat.ip6s_fragmented);
                 rrddim_set(st, "failed", ip6stat.ip6s_cantfrag);
@@ -1069,8 +999,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 rrdset_done(st);
             }
 
-            // --------------------------------------------------------------------
-
             if (do_ip6_fragsin == CONFIG_BOOLEAN_YES || (do_ip6_fragsin == CONFIG_BOOLEAN_AUTO &&
                                                          (ip6stat.ip6s_reassembled ||
                                                           ip6stat.ip6s_fragdropped ||
@@ -1100,8 +1028,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                     rrddim_add(st, "failed", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "timeout", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "all", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "ok", ip6stat.ip6s_reassembled);
                 rrddim_set(st, "failed", ip6stat.ip6s_fragdropped);
@@ -1110,8 +1037,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 rrdset_done(st);
             }
 
-            // --------------------------------------------------------------------
-
             if (do_ip6_errors == CONFIG_BOOLEAN_YES || (do_ip6_errors == CONFIG_BOOLEAN_AUTO &&
                                                         (ip6stat.ip6s_toosmall ||
                                                          ip6stat.ip6s_odropped ||
@@ -1151,8 +1076,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                     rrddim_add(st, "InNoRoutes", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
 
                     rrddim_add(st, "OutNoRoutes", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "InDiscards", ip6stat.ip6s_toosmall);
                 rrddim_set(st, "OutDiscards", ip6stat.ip6s_odropped);
@@ -1169,8 +1093,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
         }
     }
 
-    // --------------------------------------------------------------------
-
     if (likely(do_icmp6 || do_icmp6_redir || do_icmp6_errors || do_icmp6_echos || do_icmp6_router || do_icmp6_neighbor || do_icmp6_types)) {
         if (unlikely(GETSYSCTL_BY_NAME("net.inet6.icmp6.stats", icmp6stat))) {
             do_icmp6 = 0;
@@ -1205,16 +1127,13 @@ int do_macos_sysctl(int update_every, usec_t dt) {
 
                     rrddim_add(st, "received", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "sent", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "sent", icmp6_total.msgs_in);
                 rrddim_set(st, "received", icmp6_total.msgs_out);
                 rrdset_done(st);
             }
 
-            // --------------------------------------------------------------------
-
             if (do_icmp6_redir == CONFIG_BOOLEAN_YES || (do_icmp6_redir == CONFIG_BOOLEAN_AUTO &&
                                                          (icmp6stat.icp6s_inhist[ND_REDIRECT] ||
                                                           icmp6stat.icp6s_outhist[ND_REDIRECT] ||
@@ -1239,16 +1158,13 @@ int do_macos_sysctl(int update_every, usec_t dt) {
 
                     rrddim_add(st, "received", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "sent", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "sent", icmp6stat.icp6s_inhist[ND_REDIRECT]);
                 rrddim_set(st, "received", icmp6stat.icp6s_outhist[ND_REDIRECT]);
                 rrdset_done(st);
             }
 
-            // --------------------------------------------------------------------
-
             if (do_icmp6_errors == CONFIG_BOOLEAN_YES || (do_icmp6_errors == CONFIG_BOOLEAN_AUTO &&
                                                           (icmp6stat.icp6s_badcode ||
                                                            icmp6stat.icp6s_badlen ||
@@ -1291,8 +1207,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                     rrddim_add(st, "OutDestUnreachs", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutTimeExcds", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutParmProblems", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "InErrors", icmp6stat.icp6s_badcode + icmp6stat.icp6s_badlen + icmp6stat.icp6s_checksum + icmp6stat.icp6s_tooshort);
                 rrddim_set(st, "OutErrors", icmp6stat.icp6s_error);
@@ -1307,8 +1222,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 rrdset_done(st);
             }
 
-            // --------------------------------------------------------------------
-
             if (do_icmp6_echos == CONFIG_BOOLEAN_YES || (do_icmp6_echos == CONFIG_BOOLEAN_AUTO &&
                                                          (icmp6stat.icp6s_inhist[ICMP6_ECHO_REQUEST] ||
                                                           icmp6stat.icp6s_outhist[ICMP6_ECHO_REQUEST] ||
@@ -1337,8 +1250,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                     rrddim_add(st, "OutEchos", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "InEchoReplies", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutEchoReplies", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "InEchos", icmp6stat.icp6s_inhist[ICMP6_ECHO_REQUEST]);
                 rrddim_set(st, "OutEchos", icmp6stat.icp6s_outhist[ICMP6_ECHO_REQUEST]);
@@ -1347,8 +1259,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 rrdset_done(st);
             }
 
-            // --------------------------------------------------------------------
-
             if (do_icmp6_router == CONFIG_BOOLEAN_YES || (do_icmp6_router == CONFIG_BOOLEAN_AUTO &&
                                                           (icmp6stat.icp6s_inhist[ND_ROUTER_SOLICIT] ||
                                                            icmp6stat.icp6s_outhist[ND_ROUTER_SOLICIT] ||
@@ -1377,8 +1287,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                     rrddim_add(st, "OutSolicits", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "InAdvertisements", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutAdvertisements", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "InSolicits", icmp6stat.icp6s_inhist[ND_ROUTER_SOLICIT]);
                 rrddim_set(st, "OutSolicits", icmp6stat.icp6s_outhist[ND_ROUTER_SOLICIT]);
@@ -1387,8 +1296,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 rrdset_done(st);
             }
 
-            // --------------------------------------------------------------------
-
             if (do_icmp6_neighbor == CONFIG_BOOLEAN_YES || (do_icmp6_neighbor == CONFIG_BOOLEAN_AUTO &&
                                                             (icmp6stat.icp6s_inhist[ND_NEIGHBOR_SOLICIT] ||
                                                              icmp6stat.icp6s_outhist[ND_NEIGHBOR_SOLICIT] ||
@@ -1417,18 +1324,14 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                     rrddim_add(st, "OutSolicits", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "InAdvertisements", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutAdvertisements", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "InSolicits", icmp6stat.icp6s_inhist[ND_NEIGHBOR_SOLICIT]);
                 rrddim_set(st, "OutSolicits", icmp6stat.icp6s_outhist[ND_NEIGHBOR_SOLICIT]);
                 rrddim_set(st, "InAdvertisements", icmp6stat.icp6s_inhist[ND_NEIGHBOR_ADVERT]);
                 rrddim_set(st, "OutAdvertisements", icmp6stat.icp6s_outhist[ND_NEIGHBOR_ADVERT]);
-                rrdset_done(st);
             }
 
-            // --------------------------------------------------------------------
-
             if (do_icmp6_types == CONFIG_BOOLEAN_YES || (do_icmp6_types == CONFIG_BOOLEAN_AUTO &&
                                                          (icmp6stat.icp6s_inhist[1] ||
                                                           icmp6stat.icp6s_inhist[128] ||
@@ -1469,8 +1372,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                     rrddim_add(st, "OutType133", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutType135", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutType143", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
-                } else
-                    rrdset_next(st);
+                }
 
                 rrddim_set(st, "InType1", icmp6stat.icp6s_inhist[1]);
                 rrddim_set(st, "InType128", icmp6stat.icp6s_inhist[128]);
@@ -1487,8 +1389,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
         }
     }
 
-    // --------------------------------------------------------------------
-
     if (likely(do_uptime)) {
         if (unlikely(GETSYSCTL_BY_NAME("kern.boottime", boot_time))) {
             do_uptime = 0;
@@ -1514,7 +1414,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 );
                 rrddim_add(st, "uptime", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
             }
-            else rrdset_next(st);
 
             rrddim_set(st, "uptime", cur_time.tv_sec - boot_time.tv_sec);
             rrdset_done(st);
@@ -1523,4 +1422,3 @@ int do_macos_sysctl(int update_every, usec_t dt) {
 
     return 0;
 }
-
