@@ -40,11 +40,7 @@ static struct proc_module {
     {.name = "/proc/net/wireless",           .dim = "netwireless",  .func = do_proc_net_wireless},
     {.name = "/proc/net/sockstat",           .dim = "sockstat",     .func = do_proc_net_sockstat},
     {.name = "/proc/net/sockstat6",          .dim = "sockstat6",    .func = do_proc_net_sockstat6},
-    {.name = "/proc/net/netstat",
-     .dim = "netstat",
-     .func = do_proc_net_netstat}, // this has to be before /proc/net/snmp, because there is a shared metric
-    {.name = "/proc/net/snmp",               .dim = "snmp",         .func = do_proc_net_snmp},
-    {.name = "/proc/net/snmp6",              .dim = "snmp6",        .func = do_proc_net_snmp6},
+    {.name = "/proc/net/netstat",            .dim = "netstat",      .func = do_proc_net_netstat},
     {.name = "/proc/net/sctp/snmp",          .dim = "sctp",         .func = do_proc_net_sctp_snmp},
     {.name = "/proc/net/softnet_stat",       .dim = "softnet",      .func = do_proc_net_softnet_stat},
     {.name = "/proc/net/ip_vs/stats",        .dim = "ipvs",         .func = do_proc_net_ip_vs_stats},
