@@ -115,7 +115,7 @@ int do_proc_sys_devices_system_node(int update_every, usec_t dt) {
                             , RRDSET_TYPE_LINE
                     );
 
-                    rrdlabels_add(m->numastat_st->state->chart_labels, "numa_node", m->name, RRDLABEL_SRC_AUTO);
+                    rrdlabels_add(m->numastat_st->rrdlabels, "numa_node", m->name, RRDLABEL_SRC_AUTO);
 
                     rrdset_flag_set(m->numastat_st, RRDSET_FLAG_DETAIL);
 
@@ -127,7 +127,6 @@ int do_proc_sys_devices_system_node(int update_every, usec_t dt) {
                     rrddim_add(m->numastat_st, "other_node",     "other",      1, 1, RRD_ALGORITHM_INCREMENTAL);
 
                 }
-                else rrdset_next(m->numastat_st);
 
                 size_t lines = procfile_lines(m->numastat_ff), l;
                 for(l = 0; l < lines; l++) {
