@@ -4,5 +4,5 @@
 #define NETDATA_SQLITE_ACLK_NODE_H
 
 void sql_build_node_info(struct aclk_database_worker_config *wc, struct aclk_database_cmd cmd);
-void aclk_update_retention(struct aclk_database_worker_config *wc, struct aclk_database_cmd cmd);
+void sql_build_node_collectors(struct aclk_database_worker_config *wc);
 #endif //NETDATA_SQLITE_ACLK_NODE_H
