@@ -33,7 +33,7 @@ static void clean_instance_config(struct instance_config *config)
 void clean_instance(struct instance *instance)
 {
     clean_instance_config(&instance->config);
-    buffer_free(instance->labels);
+    buffer_free(instance->labels_buffer);
 
     uv_cond_destroy(&instance->cond_var);
     // uv_mutex_destroy(&instance->mutex);
