@@ -7,7 +7,7 @@ static struct engine *engine = NULL;
 void analytics_exporting_connectors_ssl(BUFFER *b)
 {
 #ifdef ENABLE_HTTPS
-    if (netdata_exporting_ctx) {
+    if (netdata_ssl_exporting_ctx) {
         for (struct instance *instance = engine->instance_root; instance; instance = instance->next) {
             struct simple_connector_data *connector_specific_data = instance->connector_specific_data;
             if (connector_specific_data->flags == NETDATA_SSL_HANDSHAKE_COMPLETE) {
