@@ -99,6 +99,7 @@ int init_mongodb_instance(struct instance *instance)
         instance->metric_formatting = format_dimension_stored_json_plaintext;
 
     instance->end_chart_formatting = NULL;
+    instance->variables_formatting = NULL;
     instance->end_host_formatting = flush_host_labels;
     instance->end_batch_formatting = format_batch_mongodb;
 
@@ -327,7 +328,7 @@ void mongodb_connector_worker(void *instance_p)
         }
 
         debug(
-            D_BACKEND,
+            D_EXPORTING,
             "EXPORTING: mongodb_insert(): destination = %s, database = %s, collection = %s, data size = %zu",
             instance->config.destination,
             connector_specific_config->database,
