@@ -20,6 +20,9 @@ void add_metric(
     const char *name, const char *chart, const char *family, const char *dimension,
     const char *instance, const double value, const int64_t timestamp);
 
+void add_variable(
+    void *write_request_p, const char *name, const char *instance, const double value, const int64_t timestamp);
+
 size_t get_write_request_size(void *write_request_p);
 
 int pack_and_clear_write_request(void *write_request_p, char *buffer, size_t *size);
