@@ -3,38 +3,17 @@
 #ifndef ML_HOST_H
 #define ML_HOST_H
 
-#include "BitRateWindow.h"
 #include "Config.h"
-#include "Database.h"
 #include "Dimension.h"
 
 #include "ml-private.h"
+#include "json/single_include/nlohmann/json.hpp"
 
 namespace ml {
 
 class RrdHost {
 public:
-    RrdHost(RRDHOST *RH) : RH(RH) {
-        AnomalyRateRS = rrdset_create(
-            RH,
-            "anomaly_detection",
-            "anomaly_rates",
-            NULL, // name
-            "anomaly_rates",
-            NULL, // ctx
-            "Average anomaly rate",
-            "anomaly rate",
-            "netdata",
-            "ml",
-            39189,
-            Cfg.DBEngineAnomalyRateEvery,
-            RRDSET_TYPE_LINE
-        );
-
-        AnomalyRateRS->flags = static_cast<RRDSET_FLAGS>(
-            static_cast<int>(AnomalyRateRS->flags) | RRDSET_FLAG_HIDDEN
-        );
-    }
+    RrdHost(RRDHOST *RH) : RH(RH) {};
 
     RRDHOST *getRH() { return RH; }
 
@@ -55,7 +34,6 @@ public:
 
 protected:
     RRDHOST *RH;
-    RRDSET *AnomalyRateRS;
 
     // Protect dimension and lock maps
     std::mutex Mutex;
@@ -80,6 +58,8 @@ public:
         memcpy(RU, &ResourceUsage, sizeof(struct rusage));
     }
 
+    void getModelsAsJson(nlohmann::json &Json);
+
 private:
     std::pair<Dimension *, Duration<double>> findDimensionToTrain(const TimePoint &NowTP);
     void trainDimension(Dimension *D, const TimePoint &NowTP);
@@ -95,16 +75,6 @@ public:
     void startAnomalyDetectionThreads();
     void stopAnomalyDetectionThreads();
 
-    template<typename ...ArgTypes>
-    bool getAnomalyInfo(ArgTypes&&... Args) {
-        return DB.getAnomalyInfo(Args...);
-    }
-
-    template<typename ...ArgTypes>
-    bool getAnomaliesInRange(ArgTypes&&... Args) {
-        return DB.getAnomaliesInRange(Args...);
-    }
-
     void getDetectionInfoAsJson(nlohmann::json &Json) const;
 
 private:
@@ -115,23 +85,12 @@ private:
     std::thread TrainingThread;
     std::thread DetectionThread;
 
-    BitRateWindow BRW{
-        static_cast<size_t>(Cfg.ADMinWindowSize),
-        static_cast<size_t>(Cfg.ADMaxWindowSize),
-        static_cast<size_t>(Cfg.ADIdleWindowSize),
-        static_cast<size_t>(Cfg.ADMinWindowSize * Cfg.ADWindowRateThreshold)
-    };
-
-    CalculatedNumber WindowAnomalyRate{0.0};
+    CalculatedNumber HostAnomalyRate{0.0};
 
     size_t NumAnomalousDimensions{0};
     size_t NumNormalDimensions{0};
     size_t NumTrainedDimensions{0};
     size_t NumActiveDimensions{0};
-
-    unsigned AnomalyRateTimer{0};
-
-    Database DB{Cfg.AnomalyDBPath};
 };
 
 using Host = DetectableHost;
