@@ -4,7 +4,8 @@
 
 if [ -z "${NETDATA_HEALTHCHECK_TARGET}" ] ; then
     # If users didn't request something else, query `/api/v1/info`.
-    NETDATA_HEALTHCHECK_TARGET="http://localhost:19999/api/v1/info"
+    PORT=${NETDATA_LISTENER_PORT:-19999}
+    NETDATA_HEALTHCHECK_TARGET="http://localhost:${PORT}/api/v1/info"
 fi
 
 case "${NETDATA_HEALTHCHECK_TARGET}" in
