@@ -7,6 +7,9 @@
 #
 # Author: Paul Emm. Katsoulakis <paul@netdata.cloud>
 
+apk update || exit 1
+apk upgrade || exit 1
+
 # Add required APK packages
 apk add --no-cache -U \
   alpine-sdk \
@@ -21,6 +24,7 @@ apk add --no-cache -U \
   git \
   gnutls-dev \
   gzip \
+  jq \
   libelf-static \
   libmnl-dev \
   libnetfilter_acct-dev \
