@@ -8,13 +8,13 @@
 // single exponential smoothing
 
 struct grouping_des {
-    calculated_number alpha;
-    calculated_number alpha_other;
-    calculated_number beta;
-    calculated_number beta_other;
+    NETDATA_DOUBLE alpha;
+    NETDATA_DOUBLE alpha_other;
+    NETDATA_DOUBLE beta;
+    NETDATA_DOUBLE beta_other;
 
-    calculated_number level;
-    calculated_number trend;
+    NETDATA_DOUBLE level;
+    NETDATA_DOUBLE trend;
 
     size_t count;
 };
@@ -31,22 +31,22 @@ void grouping_init_des(void) {
     }
 }
 
-static inline calculated_number window(RRDR *r, struct grouping_des *g) {
+static inline NETDATA_DOUBLE window(RRDR *r, struct grouping_des *g) {
     (void)g;
 
-    calculated_number points;
+    NETDATA_DOUBLE points;
     if(r->group == 1) {
         // provide a running DES
-        points = r->internal.points_wanted;
+        points = (NETDATA_DOUBLE)r->internal.points_wanted;
     }
     else {
         // provide a SES with flush points
-        points = r->group;
+        points = (NETDATA_DOUBLE)r->group;
     }
 
     // https://en.wikipedia.org/wiki/Moving_average#Exponential_moving_average
     // A commonly used value for alpha is 2 / (N + 1)
-    return (points > max_window_size) ? max_window_size : points;
+    return (points > (NETDATA_DOUBLE)max_window_size) ? (NETDATA_DOUBLE)max_window_size : points;
 }
 
 static inline void set_alpha(RRDR *r, struct grouping_des *g) {
@@ -69,14 +69,14 @@ static inline void set_beta(RRDR *r, struct grouping_des *g) {
     //info("beta for chart '%s' is " CALCULATED_NUMBER_FORMAT, r->st->name, g->beta);
 }
 
-void *grouping_create_des(RRDR *r) {
-    struct grouping_des *g = (struct grouping_des *)malloc(sizeof(struct grouping_des));
+void grouping_create_des(RRDR *r, const char *options __maybe_unused) {
+    struct grouping_des *g = (struct grouping_des *)onewayalloc_mallocz(r->internal.owa, sizeof(struct grouping_des));
     set_alpha(r, g);
     set_beta(r, g);
     g->level = 0.0;
     g->trend = 0.0;
     g->count = 0;
-    return g;
+    r->internal.grouping_data = g;
 }
 
 // resets when switches dimensions
@@ -92,43 +92,41 @@ void grouping_reset_des(RRDR *r) {
 }
 
 void grouping_free_des(RRDR *r) {
-    freez(r->internal.grouping_data);
+    onewayalloc_freez(r->internal.owa, r->internal.grouping_data);
     r->internal.grouping_data = NULL;
 }
 
-void grouping_add_des(RRDR *r, calculated_number value) {
+void grouping_add_des(RRDR *r, NETDATA_DOUBLE value) {
     struct grouping_des *g = (struct grouping_des *)r->internal.grouping_data;
 
-    if(calculated_number_isnumber(value)) {
-        if(likely(g->count > 0)) {
-            // we have at least a number so far
+    if(likely(g->count > 0)) {
+        // we have at least a number so far
 
-            if(unlikely(g->count == 1)) {
-                // the second value we got
-                g->trend = value - g->trend;
-                g->level = value;
-            }
-
-            // for the values, except the first
-            calculated_number last_level = g->level;
-            g->level = (g->alpha * value) + (g->alpha_other * (g->level + g->trend));
-            g->trend = (g->beta * (g->level - last_level)) + (g->beta_other * g->trend);
-        }
-        else {
-            // the first value we got
-            g->level = g->trend = value;
+        if(unlikely(g->count == 1)) {
+            // the second value we got
+            g->trend = value - g->trend;
+            g->level = value;
         }
 
-        g->count++;
+        // for the values, except the first
+        NETDATA_DOUBLE last_level = g->level;
+        g->level = (g->alpha * value) + (g->alpha_other * (g->level + g->trend));
+        g->trend = (g->beta * (g->level - last_level)) + (g->beta_other * g->trend);
     }
+    else {
+        // the first value we got
+        g->level = g->trend = value;
+    }
+
+    g->count++;
 
     //fprintf(stderr, "value: " CALCULATED_NUMBER_FORMAT ", level: " CALCULATED_NUMBER_FORMAT ", trend: " CALCULATED_NUMBER_FORMAT "\n", value, g->level, g->trend);
 }
 
-calculated_number grouping_flush_des(RRDR *r, RRDR_VALUE_FLAGS *rrdr_value_options_ptr) {
+NETDATA_DOUBLE grouping_flush_des(RRDR *r, RRDR_VALUE_FLAGS *rrdr_value_options_ptr) {
     struct grouping_des *g = (struct grouping_des *)r->internal.grouping_data;
 
-    if(unlikely(!g->count || !calculated_number_isnumber(g->level))) {
+    if(unlikely(!g->count || !netdata_double_isnumber(g->level))) {
         *rrdr_value_options_ptr |= RRDR_VALUE_EMPTY;
         return 0.0;
     }
