@@ -6,13 +6,12 @@
 // max
 
 struct grouping_max {
-    calculated_number max;
+    NETDATA_DOUBLE max;
     size_t count;
 };
 
-void *grouping_create_max(RRDR *r) {
-    (void)r;
-    return callocz(1, sizeof(struct grouping_max));
+void grouping_create_max(RRDR *r, const char *options __maybe_unused) {
+    r->internal.grouping_data = onewayalloc_callocz(r->internal.owa, 1, sizeof(struct grouping_max));
 }
 
 // resets when switches dimensions
@@ -24,25 +23,23 @@ void grouping_reset_max(RRDR *r) {
 }
 
 void grouping_free_max(RRDR *r) {
-    freez(r->internal.grouping_data);
+    onewayalloc_freez(r->internal.owa, r->internal.grouping_data);
     r->internal.grouping_data = NULL;
 }
 
-void grouping_add_max(RRDR *r, calculated_number value) {
-    if(!isnan(value)) {
-        struct grouping_max *g = (struct grouping_max *)r->internal.grouping_data;
+void grouping_add_max(RRDR *r, NETDATA_DOUBLE value) {
+    struct grouping_max *g = (struct grouping_max *)r->internal.grouping_data;
 
-        if(!g->count || calculated_number_fabs(value) > calculated_number_fabs(g->max)) {
-            g->max = value;
-            g->count++;
-        }
+    if(!g->count || fabsndd(value) > fabsndd(g->max)) {
+        g->max = value;
+        g->count++;
     }
 }
 
-calculated_number grouping_flush_max(RRDR *r, RRDR_VALUE_FLAGS *rrdr_value_options_ptr) {
+NETDATA_DOUBLE grouping_flush_max(RRDR *r, RRDR_VALUE_FLAGS *rrdr_value_options_ptr) {
     struct grouping_max *g = (struct grouping_max *)r->internal.grouping_data;
 
-    calculated_number value;
+    NETDATA_DOUBLE value;
 
     if(unlikely(!g->count)) {
         value = 0.0;
