@@ -6,13 +6,12 @@
 // min
 
 struct grouping_min {
-    calculated_number min;
+    NETDATA_DOUBLE min;
     size_t count;
 };
 
-void *grouping_create_min(RRDR *r) {
-    (void)r;
-    return callocz(1, sizeof(struct grouping_min));
+void grouping_create_min(RRDR *r, const char *options __maybe_unused) {
+    r->internal.grouping_data = onewayalloc_callocz(r->internal.owa, 1, sizeof(struct grouping_min));
 }
 
 // resets when switches dimensions
@@ -24,25 +23,23 @@ void grouping_reset_min(RRDR *r) {
 }
 
 void grouping_free_min(RRDR *r) {
-    freez(r->internal.grouping_data);
+    onewayalloc_freez(r->internal.owa, r->internal.grouping_data);
     r->internal.grouping_data = NULL;
 }
 
-void grouping_add_min(RRDR *r, calculated_number value) {
-    if(!isnan(value)) {
-        struct grouping_min *g = (struct grouping_min *)r->internal.grouping_data;
+void grouping_add_min(RRDR *r, NETDATA_DOUBLE value) {
+    struct grouping_min *g = (struct grouping_min *)r->internal.grouping_data;
 
-        if(!g->count || calculated_number_fabs(value) < calculated_number_fabs(g->min)) {
-            g->min = value;
-            g->count++;
-        }
+    if(!g->count || fabsndd(value) < fabsndd(g->min)) {
+        g->min = value;
+        g->count++;
     }
 }
 
-calculated_number grouping_flush_min(RRDR *r, RRDR_VALUE_FLAGS *rrdr_value_options_ptr) {
+NETDATA_DOUBLE grouping_flush_min(RRDR *r, RRDR_VALUE_FLAGS *rrdr_value_options_ptr) {
     struct grouping_min *g = (struct grouping_min *)r->internal.grouping_data;
 
-    calculated_number value;
+    NETDATA_DOUBLE value;
 
     if(unlikely(!g->count)) {
         value = 0.0;
