@@ -7,9 +7,9 @@
 // single exponential smoothing
 
 struct grouping_ses {
-    calculated_number alpha;
-    calculated_number alpha_other;
-    calculated_number level;
+    NETDATA_DOUBLE alpha;
+    NETDATA_DOUBLE alpha_other;
+    NETDATA_DOUBLE level;
     size_t count;
 };
 
@@ -25,20 +25,20 @@ void grouping_init_ses(void) {
     }
 }
 
-static inline calculated_number window(RRDR *r, struct grouping_ses *g) {
+static inline NETDATA_DOUBLE window(RRDR *r, struct grouping_ses *g) {
     (void)g;
 
-    calculated_number points;
+    NETDATA_DOUBLE points;
     if(r->group == 1) {
         // provide a running DES
-        points = r->internal.points_wanted;
+        points = (NETDATA_DOUBLE)r->internal.points_wanted;
     }
     else {
         // provide a SES with flush points
-        points = r->group;
+        points = (NETDATA_DOUBLE)r->group;
     }
 
-    return (points > max_window_size) ? max_window_size : points;
+    return (points > (NETDATA_DOUBLE)max_window_size) ? (NETDATA_DOUBLE)max_window_size : points;
 }
 
 static inline void set_alpha(RRDR *r, struct grouping_ses *g) {
@@ -48,11 +48,11 @@ static inline void set_alpha(RRDR *r, struct grouping_ses *g) {
     g->alpha_other = 1.0 - g->alpha;
 }
 
-void *grouping_create_ses(RRDR *r) {
-    struct grouping_ses *g = (struct grouping_ses *)callocz(1, sizeof(struct grouping_ses));
+void grouping_create_ses(RRDR *r, const char *options __maybe_unused) {
+    struct grouping_ses *g = (struct grouping_ses *)onewayalloc_callocz(r->internal.owa, 1, sizeof(struct grouping_ses));
     set_alpha(r, g);
     g->level = 0.0;
-    return g;
+    r->internal.grouping_data = g;
 }
 
 // resets when switches dimensions
@@ -64,26 +64,24 @@ void grouping_reset_ses(RRDR *r) {
 }
 
 void grouping_free_ses(RRDR *r) {
-    freez(r->internal.grouping_data);
+    onewayalloc_freez(r->internal.owa, r->internal.grouping_data);
     r->internal.grouping_data = NULL;
 }
 
-void grouping_add_ses(RRDR *r, calculated_number value) {
+void grouping_add_ses(RRDR *r, NETDATA_DOUBLE value) {
     struct grouping_ses *g = (struct grouping_ses *)r->internal.grouping_data;
 
-    if(calculated_number_isnumber(value)) {
-        if(unlikely(!g->count))
-            g->level = value;
+    if(unlikely(!g->count))
+        g->level = value;
 
-        g->level = g->alpha * value + g->alpha_other * g->level;
-        g->count++;
-    }
+    g->level = g->alpha * value + g->alpha_other * g->level;
+    g->count++;
 }
 
-calculated_number grouping_flush_ses(RRDR *r, RRDR_VALUE_FLAGS *rrdr_value_options_ptr) {
+NETDATA_DOUBLE grouping_flush_ses(RRDR *r, RRDR_VALUE_FLAGS *rrdr_value_options_ptr) {
     struct grouping_ses *g = (struct grouping_ses *)r->internal.grouping_data;
 
-    if(unlikely(!g->count || !calculated_number_isnumber(g->level))) {
+    if(unlikely(!g->count || !netdata_double_isnumber(g->level))) {
         *rrdr_value_options_ptr |= RRDR_VALUE_EMPTY;
         return 0.0;
     }
