@@ -36,10 +36,10 @@ extern int             web_allow_mgmt_dns;
 
 extern WEB_SERVER_MODE web_server_mode;
 
-extern WEB_SERVER_MODE web_server_mode_id(const char *mode);
-extern const char *web_server_mode_name(WEB_SERVER_MODE id);
+WEB_SERVER_MODE web_server_mode_id(const char *mode);
+const char *web_server_mode_name(WEB_SERVER_MODE id);
 
-extern void api_listen_sockets_setup(void);
+void api_listen_sockets_setup(void);
 
 #define DEFAULT_TIMEOUT_TO_RECEIVE_FIRST_WEB_REQUEST 60
 #define DEFAULT_DISCONNECT_IDLE_WEB_CLIENTS_AFTER_SECONDS 60
@@ -49,10 +49,10 @@ extern long web_client_streaming_rate_t;
 
 #ifdef WEB_SERVER_INTERNALS
 extern LISTEN_SOCKETS api_sockets;
-extern void web_client_update_acl_matches(struct web_client *w);
-extern void web_server_log_connection(struct web_client *w, const char *msg);
-extern void web_client_initialize_connection(struct web_client *w);
-extern struct web_client *web_client_create_on_listenfd(int listener);
+void web_client_update_acl_matches(struct web_client *w);
+void web_server_log_connection(struct web_client *w, const char *msg);
+void web_client_initialize_connection(struct web_client *w);
+struct web_client *web_client_create_on_listenfd(int listener);
 
 #include "web_client_cache.h"
 #endif // WEB_SERVER_INTERNALS
